@@ -63,7 +63,7 @@ if [ ".$view" != . ]; then
         echo "______________________________________________________________________"
         $openssl x509 -noout -text -in $sslcrtdir/ca.crt
         echo ""
-        if [ ".`$openssl x509 -noout -text -in $sslcrtdir/ca.crt | grep "Signature Algorithm" | grep -i RSA`" != . ]; then
+        if [ ".`$openssl x509 -noout -text -in $sslcrtdir/ca.crt | grep 'Signature Algorithm' | grep -i RSA`" != . ]; then
             echo "${T_MD}CA RSA Private Key${T_ME} [ca.key]"
             echo "______________________________________________________________________"
             $openssl rsa -noout -text -in $sslkeydir/ca.key
@@ -79,7 +79,7 @@ if [ ".$view" != . ]; then
         echo "______________________________________________________________________"
         $openssl x509 -noout -text -in $sslcrtdir/server.crt
         echo ""
-        if [ ".`$openssl x509 -noout -text -in $sslcrtdir/server.crt | grep "Signature Algorithm" | grep -i RSA`" != . ]; then
+        if [ ".`$openssl x509 -noout -text -in $sslcrtdir/server.crt | grep 'Signature Algorithm' | grep -i RSA`" != . ]; then
             echo "${T_MD}Server RSA Private Key${T_ME} [server.key]"
             echo "______________________________________________________________________"
             $openssl rsa -noout -text -in $sslkeydir/server.key
